@@ -22,6 +22,7 @@ setAs("dsCMatrix", "dgCMatrix",
 for(prefix in c("d", "l", "n"))
     setAs(paste0(prefix,"sCMatrix"), "generalMatrix",
 	  function(from) .Call(Csparse_symmetric_to_general, from))
+rm(prefix)
 
 setAs("dtCMatrix", "dtTMatrix",
       function(from) .Call(Csparse_to_Tsparse, from, TRUE))
@@ -48,25 +49,31 @@ setAs("CsparseMatrix", "denseMatrix",
 setAs("dgCMatrix", "dgeMatrix",
       function(from) .Call(Csparse_to_dense, from))
 
+setAs("dgCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
+setAs("dsCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
+setMethod("as.vector", signature(x = "dgCMatrix", mode = "missing"),
+	  function(x, mode) .Call(Csparse_to_vector, x))
+setMethod("as.vector", signature(x = "dsCMatrix", mode = "missing"),
+	  function(x, mode) .Call(Csparse_to_vector, x))
+## could do these and more for as(., "numeric") ... but we *do* recommend  as(*,"vector"):
+## setAs("dgCMatrix", "numeric", Csp2vec)
+## setAs("dsCMatrix", "numeric", Csp2vec)
+
+## |-> cholmod_C -> cholmod_dense -> chm_dense_to_matrix
 ## cholmod_sparse_to_dense converts symmetric storage to general
 ## storage so symmetric classes are ok for conversion to matrix.
 ## unit triangular needs special handling
-setAs("CsparseMatrix", "matrix",
-      function(from) {
-          ## |-> cholmod_C -> cholmod_dense -> chm_dense_to_matrix
-	  .Call(Csparse_to_matrix, .Call(Csparse_diagU2N, from))
-      })
+setAs("dgCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from))
+setAs("dsCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from))
+setAs("dtCMatrix", "matrix", function(from)
+      .Call(Csparse_to_matrix, .Call(Csparse_diagU2N, from)))
+## NB: Would *not* be ok for l*Matrix or n*Matrix,
+## --------- as cholmod coerces to "REAL" aka "double"
 
 setAs("CsparseMatrix", "symmetricMatrix",
       function(from) {
-	  if(isSymmetric(from)) {
-	      isTri <- is(from, "triangularMatrix")# i.e. effectively *diagonal*
-	      if (isTri && from@diag == "U")
-		  from <- .Call(Csparse_diagU2N, from)
-	      .Call(Csparse_general_to_symmetric, from,
-		    uplo = if(isTri) from@uplo else "U")
-	  } else
-	  stop("not a symmetric matrix; consider forceSymmetric() or symmpart()")
+	  if(isSymmetric(from)) forceCspSymmetric(from)
+	  else stop("not a symmetric matrix; consider forceSymmetric() or symmpart()")
       })
 
 
@@ -74,6 +81,8 @@ setAs("CsparseMatrix", "symmetricMatrix",
     .Call(Csparse_validate2, x, sort.if.needed)
 ##-> to be used in sparseMatrix(.), e.g. --- but is unused currently
 ## NB: 'sort.if.needed' is called 'maybe_modify' in C -- so be careful
+## more useful:
+.sortCsparse <- function(x) .Call(Csparse_sort, x) ## modifies 'x' !!
 
 ### Some group methods:
 
@@ -199,7 +208,7 @@ setMethod("[", signature(x = "CsparseMatrix",
 
 
 
-## workhorse for "[<-" -- both for d* and l*  C-sparse matrices :
+## workhorse for "[<-" -- for d*, l*, and n..C-sparse matrices :
 ## ---------     -----
 replCmat <- function (x, i, j, ..., value)
 {
@@ -215,8 +224,9 @@ replCmat <- function (x, i, j, ..., value)
     if(na == 3) { ## "vector (or 2-col) indexing"  M[i] <- v
 	x <- as(x, "TsparseMatrix")
 	x[i] <- value # may change class e.g. from dtT* to dgT*
-	clx <- sub(".Matrix$", "CMatrix", class(x))
-	if(any(is0(x@x))) ## drop all values that "happen to be 0"
+	clx <- sub(".Matrix$", "CMatrix", (c.x <- class(x)))
+	if("x" %in% .slotNames(c.x) && any0(x@x))
+	    ## drop all values that "happen to be 0"
 	    drop0(x, is.Csparse=FALSE) else as_CspClass(x, clx)
     }
     else ## nargs() == 4 :
@@ -345,7 +355,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
 	    has0 <-
 		if(spV) length(value@i) < lenV else any(value[!is.na(value)] == 0)
 	    if(lenV < lenRepl)
-		value <- rep(value, length = lenRepl)
+		value <- rep_len(value, lenRepl)
 	    ## Ideally we only replace them where value != 0 and drop the value==0
 	    ## ones; FIXME: see Davis(2006) "2.7 Removing entries", p.16, e.g. use cs_dropzeros()
 	    ##	     but really could be faster and write something like cs_drop_k(A, k)
@@ -375,7 +385,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
 	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
     }# else{ not using new memory-sparse  code
-    if(has.x && any(is0(x@x))) ## drop all values that "happen to be 0"
+    if(has.x && any0(x@x)) ## drop all values that "happen to be 0"
 	as_CspClass(drop0(x), clx)
     else as_CspClass(x, clx)
 } ## replCmat4
@@ -414,6 +424,13 @@ setReplaceMethod("[", signature(x = "CsparseMatrix", i = "matrix", j = "missing"
 		    "CsparseMatrix"))
 ## more in ./sparseMatrix.R (and ./Matrix.R )
 
+setReplaceMethod("[", signature(x = "CsparseMatrix", i = "Matrix", j = "missing",
+				value = "replValue"),
+		 function(x, i, j, ..., value)
+		 ## goto Tsparse modify and convert back:
+		 as(.TM.repl.i.mat(as(x, "TsparseMatrix"), i=i, value=value),
+		    "CsparseMatrix"))
+
 
 setMethod("t", signature(x = "CsparseMatrix"),
 	  function(x) .Call(Csparse_transpose, x, is(x, "triangularMatrix")))
